#!/usr/bin/perl -w

package Test::Repocop::Common;
use Test::Repocop::Options;
use File::Basename;

use strict;

sub rpmfile2key {
    $_=shift;
    s!^.*/!!;
    s/\.rpm$//;
    return $_;
}

sub set_test_environment {
    my ($testname)=@_;
    # todo: remove dublicates from repocop-run
    my $repocop_test_statedir="$repocop_cachedir/testcache";
    my $repocop_test_tmpdir="$repocop_cachedir/tmp";
    $ENV{'REPOCOP_STATEDIR'}=$repocop_test_statedir;
    $ENV{'REPOCOP_TEST_STATEDIR'}="$repocop_test_statedir/$testname";
    $ENV{'REPOCOP_TEST_TMPDIR'}="$repocop_test_tmpdir/$testname";
    $ENV{'REPOCOP_TEST_DBDIR'}=$repocop_test_dbdir;
    $ENV{'REPOCOP_TEST_NAME'}=$testname;
    my $repocop_test_db="$repocop_test_dbdir/$testname.db";
    $ENV{'REPOCOP_TEST_DB'}=$repocop_test_db if -e $repocop_test_db;
}

sub glob_tests {
    my ($file,@dirs)=@_;
    my @retval;
    foreach (@dirs) {
	push @retval, glob "$_/*/$file";
    }
    return @retval;
}

sub filepath2testname {
    return basename(dirname($_[0]));
}

#sub is_srpm {
#    return $_[0]=~/\.src$/;
#}

sub find_test_dir {
    my ($testname)=@_;
    my @tests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/{pkgtests,srctests,pkgcollectors,srccollectors}");
    foreach my $dir(@tests_dir) {
	$dir .= '/'.$testname;
	return $dir if -d $dir;
    }
}

1;

=head1	NAME

Test::Repocop::Common - common private subroutines for repocop.

=head1	DESCRIPTION

Common private subroutines for repocop. 
The interface is subject to change without notice.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop. 

=head1	COPYING

Copyright (c) 2008 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

