#!/usr/bin/perl -w

package Test::Repocop::Options;
require Exporter;
our @ISA = qw(Exporter);

our @EXPORT = qw(
$repocop_cachedir $repocop_test_outputdir $repocop_metadatadir 
$repocop_test_dbdir $verbose &repocop_note
);
our @EXPORT_OK = qw();

push @EXPORT_OK, @EXPORT;
use strict;
use Getopt::Long;
use Pod::Usage;

our $repocop_cachedir="$ENV{HOME}/.repocop";
$repocop_cachedir=$ENV{'REPOCOP_CACHEDIR'} if $ENV{'REPOCOP_CACHEDIR'};

our ($repocop_test_outputdir,$repocop_metadatadir,$repocop_test_dbdir,$verbose);

my (@excludetest, @includetest, @excludepackager, @includepackager);
my ($help, $lastrun, $newer);
my @pkgtests_dirarg;
my @srctests_dirarg;
my @pkgcollectors_dirarg;
my @srccollectors_dirarg;

my $rundir=`pwd`;
chomp $rundir;

sub die_if_nothing_to_report {
    die "cachedir $repocop_cachedir does not exist or is not a directory!\n" if ( ! -d "$repocop_cachedir");
    die "Results of tests are not found in $repocop_cachedir - nothing to do. run repocop-run first!\n" if (! -d "$repocop_test_outputdir");
}

sub repocop_note {
    print "$_[0]\n" if $verbose;
}

sub get_common_options {
    GetOptions (
	"cachedir=s" => \$repocop_cachedir,
	"help"  => \$help,
	"report=s" => \$Repocop::arg::reportlevel,
	"except"  => \$Repocop::arg::except,
	"given"  => sub { $Repocop::arg::except=0 },
	"et|exclude-test=s"  => \@excludetest,
	"it|include-test=s"  => \@includetest,
	"ep|exclude-packager=s"  => \@excludepackager,
	"ip|include-packager=s"  => \@includepackager,
	"last-run|lastrun"  => \$lastrun,
	"newer=s"  => \$newer,
	"verbose+"  => \$verbose,
	'quiet'   => sub { $verbose = 0 },
	"acl-file=s" => \$Repocop::arg::aclfile,
	"pkgtests-dir=s"  => \@pkgtests_dirarg,
	"srctests-dir=s"  => \@srctests_dirarg,
	"pkgcollectors-dir=s"  => \@pkgcollectors_dirarg,
	"srccollectors-dir=s"  => \@srccollectors_dirarg,
	@_
    );
    if ($help) {
	#exec "pod2usage --exit=0 $0";
	pod2usage({ #-message => "the options below are package-specific:" ,
		    -exitval => 0  ,
		    -verbose => $verbose,
		    #-output  => $filehandle
		  } );
    }
    %Repocop::arg::EXCLUDETEST=map {$_=>1} map {split ','} @excludetest;
    %Repocop::arg::INCLUDETEST=map {$_=>1} map {split ','} @includetest;
    %Repocop::arg::EXCLUDEPACKAGER=map {$_=>1} map {split ','} @excludepackager;
    %Repocop::arg::INCLUDEPACKAGER=map {$_=>1} map {split ','} @includepackager;

    my $rundir=`pwd`;
    chomp $rundir;
    $repocop_cachedir=$rundir.'/'.$repocop_cachedir unless $repocop_cachedir=~m!^/!;
    $repocop_test_outputdir="$repocop_cachedir/tests";
    $repocop_metadatadir="$repocop_cachedir/metadata";
    $repocop_test_dbdir="$repocop_cachedir/db";

    if ($lastrun) {
	my $repocop_lastrun_file="$repocop_metadatadir/last-run";
	die "last run results not found. internal error?\n" unless -e $repocop_lastrun_file;
	my $f;
	open METADATA, $repocop_lastrun_file || die $!;
	while ($f=<METADATA>) {
	    chomp $f;
	    $Repocop::arg::pkgtable{$f}=1 if $f;
	}
	close METADATA;
    }

    foreach my $rpmarg (@ARGV) {
	if ( -d "$rpmarg") {
	    open (RPMARGS_FILES, "ls -1 $rpmarg |grep -x '[^.].*[.]rpm' |") || die $!;
	    my $f;
	    while ($f=<RPMARGS_FILES>) {
		chomp $f;
		&_add_file_to_repocop_arg_pkg("$rpmarg/$f") if -e "$rpmarg/$f";
	    }
	    close RPMARGS_FILES;
	} else {
	    &_add_file_to_repocop_arg_pkg($rpmarg);
	}
    }

    $Repocop::arg::report{'fail'}=1;
    unless ($Repocop::arg::reportlevel =~ m'^f') {
	$Repocop::arg::report{'warn'}=1;
	unless ($Repocop::arg::reportlevel  =~ m'^w') {
	    $Repocop::arg::report{'info'}=1;
	    unless ($Repocop::arg::reportlevel  =~ m'^i') {
		$Repocop::arg::report{'experimental'}=1;
		unless ($Repocop::arg::reportlevel  =~ m'^e') {
		    $Repocop::arg::report{'ok'}=1;
		    unless ($Repocop::arg::reportlevel  =~ m'^o') {
			$Repocop::arg::report{'skip'}=1;
			unless ($Repocop::arg::reportlevel  =~ m'^s') {
			    warn "strange value for --report option: $Repocop::arg::reportlevel\n";
			    $Repocop::arg::report{'skip'}=0;
			    $Repocop::arg::report{'ok'}=0;
			}
		    }
		}
	    }
	}
    }

    die "argument to --newer option, file $newer not found.\n" if $newer && ! -e $newer;

    my @pkgtests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/pkgtests");
    my @srctests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/srctests");
    my @pkgcollectors_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/pkgcollectors");
    my @srccollectors_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/srccollectors");

    push @pkgtests_dir, &_filtertestsdir(@pkgtests_dirarg);
    push @srctests_dir, &_filtertestsdir(@srctests_dirarg);
    push @pkgcollectors_dir, &_filtertestsdir(@pkgcollectors_dirarg);
    push @srccollectors_dir, &_filtertestsdir(@srccollectors_dirarg);
    @Repocop::arg::tests_dir=(@pkgcollectors_dir,@srccollectors_dir,@pkgtests_dir,@srctests_dir);
    @Repocop::arg::pkgscripts_dir=(@pkgcollectors_dir,@pkgtests_dir);
    @Repocop::arg::srcscripts_dir=(@srccollectors_dir,@srctests_dir);
}

sub _add_file_to_repocop_arg_pkg {
    foreach (@_) {
	next if ($newer and -M $newer < -M $_);
	push @Repocop::arg::pkglist, $_;
	s!^.*/!!;
	s/\.rpm$//;
	$Repocop::arg::pkgtable{$_}=1;
    }
}

sub _filtertestsdir {
    my @retval;
    foreach my $dirarg (@_) {
	foreach (split (/[:,]/,$dirarg)) {
	    next unless $_;
	    $_=$rundir."/".$_ unless m!^/!;
	    if (-d $_) {
		push @retval, $_;
	    } else {
		warn "warning: argument to --{pkg,src}tests-dir: $_ is not a directory!\n";
	    }
	}
    }
    return @retval;
}

package Repocop::arg;

our $reportlevel='info';
our (@pkglist, %pkgtable, %report, $except, $aclfile);
our (%EXCLUDETEST,%INCLUDETEST,%EXCLUDEPACKAGER,%INCLUDEPACKAGER);
our (@tests_dir,@pkgscripts_dir,@srcscripts_dir);

sub pkg_keylist {
    return keys(%Repocop::arg::pkgtable);
}

sub skip_pkg {
    return 0 unless %Repocop::arg::pkgtable;
    if ($Repocop::arg::pkgtable{$_[0]}) {
	return 1 if $Repocop::arg::except;
    } else {
	return 1 unless $Repocop::arg::except; # -given mode
    }
    return 0;
}

sub skip_test {
    return 1 if %EXCLUDETEST and $EXCLUDETEST{$_[0]};
    return 1 if %INCLUDETEST and !$INCLUDETEST{$_[0]};
    return 0;
}

sub skip_nick {
    return 1 if %EXCLUDEPACKAGER and $EXCLUDEPACKAGER{$_[0]};
    return 1 if %INCLUDEPACKAGER and !$INCLUDEPACKAGER{$_[0]};
    return 0;
}

sub skip_status {
    return 1 if %Repocop::arg::report and !$Repocop::arg::report{$_[0]};
    return 0;
}

1;

=head1	NAME

Test::Repocop::Common - common options for repocop

=head1	SYNOPSIS

B<repocop-*>
[B<-h|--help>]
[B<-v|--verbose>]
[B<-q|--quiet>]
[B<-c|--cachedir> I<cachedir>]
[B<--et|--exclude-test> I<comma separated list of tests>]
[B<--it|--include-test> I<comma separated list of tests>]
[B<--ep|--exclude-packager> I<comma separated list of packager's nicks>]
[B<--ip|--include-packager> I<comma separated list of packager's nicks>]
[B<--pkgcollectors-dir> I<comma separated list of local collectors' dirs>]
[B<--srccollectors-dir> I<comma separated list of local collectors' dirs>]
[B<--pkgtests-dir> I<comma separated list of local tests' dirs>]
[B<--srctests-dir> I<comma separated list of local tests' dirs>]
[B<--ex|--except>] 
[B<-g|--given>] 
[B<-l|--last-run>] 
[B<--newer>] I<filename>
[B<-r|--report> <s[kip]|o[k]|w[arn]|f[ail]>]
[I<DIR>...] [I<FILE>...]

=head1	DESCRIPTION

B<repocop-*> processes results of repocop unit tests, created with 
repocop-run command, stored in <cachedir>.
Presize subset of tests can be selected using B<--include>
and B<--exclude> options.

=head1	OPTIONS

=over

=item	B<-r, --report> I<skip|experimental|ok|warn|fail>]

The level of test results reported. Test results below this level
are not reported. Default is warn.

=item	B<-c,--cachedir> I<dir>

Provides alternative location for cachedir. 
Repocop cachedir is a place where test results and 
packages metadata information are stored.

=item	B<--except>, B<--given>

Control processing of rpm arguments. 
B<--given> (default) means processing only given rpm arguments.
B<--except>  means processing all data except given rpm arguments.

=item	B<--et, --exclude-test> I<comma separated list of tests>

Report all processed tests exept the given excluded set.

=item	B<--it, --include-test> I<comma separated list of tests>

Report the given set of tests.

=item	B<--ep, --exclude-packager> I<comma separated list of tests>

=item	B<--it, --include-packager> I<comma separated list of tests>

Exclude/include packages according to Packager: tag.

=item [B<--pkgcollectors-dir> I<comma separated list of local collectors' dirs>]

=item [B<--srccollectors-dir> I<comma separated list of local collectors' dirs>]

=item [B<--pkgtests-dir> I<comma separated list of local tests' dirs>]

=item [B<--srctests-dir> I<comma separated list of local tests' dirs>]

Append user's local tests and collectors to repocop.

=item	B<-h, --help>

Display this help and exit.

=item	B<-v, --verbose>, B<-q, --quiet>

Verbosity level. Multiple -v increase the verbosity level, -q sets it to 0.

=item	B<-l, --last-run>

Use the set of packages processed at last run as an argument.

=item	B<--newer> I<filename>

Process packages newer then I<filename> only.
Note: this filtering does not apply to B<--last-run> option.

=item	B<--acl-file> I<file>

the argument is /path/to/Sisyphus/files/list/list.src.classic
This option is ALTLinux-specific. The file content is ACL db,
which is used to sort result by ALTLinux ACL.


=back

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop. 

=head1	COPYING

Copyright (c) 2008 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

