#!/usr/bin/perl -w

package Test::Repocop::TestCache;
use strict;
use DBI;
use Test::Repocop::Options; # just for cachedir, metadata && verbose :(
use Test::Repocop::Metadata;

# static: only to be called Test::Repocop::TestCache::remove_test
sub remove_test {
    return unless @_;
    my @findclause;
    foreach my $testname (@_) {
	push @findclause, '-name',$testname,'-or','-name',$testname.'+++*', '-or';
    }
    pop @findclause if @findclause;
    system ('find',$repocop_test_outputdir,'-type','f','(',@findclause,')','-delete') == 0 || die "error: $!";
}

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
	CUR_TEST_FILENAME => '',
	FILTER_NICK=> 0,
    };
    bless $self, $class;
    $self->initialize();
    return $self;
}

sub initialize {
    my $self=shift;
    open ($self->{FD}, " find $repocop_test_outputdir -type f -name '*+++*' |");
    if (scalar %Repocop::arg::EXCLUDEPACKAGER or scalar %Repocop::arg::INCLUDEPACKAGER) {
	$self->{METADATA}=Test::Repocop::Metadata->new();
	$self->{FILTER_NICK}=1;
    }
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate3 {
    my $self=shift;
    $_=readline $self->{FD};
    return unless $_;
    chomp $_;
    $self->{CUR_TEST_FILENAME}=$_;
    s!^$repocop_test_outputdir/?!!o;
    #print "$_\n";
    my ($rpmkey,$teststatus)=split '/';
    my ($test,$status)=split '\+\+\+', $teststatus;
    return ($rpmkey,$test,$status);
}

sub iterate3_filtered {
    my $self=shift;
    my ($rpm,$test,$status);
    do {
	($rpm,$test,$status)=$self->iterate3();
    } while $rpm && (&Repocop::arg::skip_test($test) || &Repocop::arg::skip_pkg($rpm) || &Repocop::arg::skip_status($status) || ($self->{FILTER_NICK} and &Repocop::arg::skip_nick($self->{METADATA}->nick($rpm))) );
    return unless $rpm;
    return ($rpm,$test,$status);
}


sub iterate4_filtered {
    my $self=shift;
    my ($rpm,$test,$status)=$self->iterate3_filtered();
    return unless $rpm;
    my $result=$self->result();
    return ($rpm,$test,$status,$result);
}

sub result {
    my $self=shift;
    return &_read_file($self->{CUR_TEST_FILENAME});
}


sub _read_file {
    my $filename=shift; 
    return "" unless -s $filename;
    open my $fh, $filename or die $!;
    local $/; # enable localized slurp mode
    my $content = <$fh>;
    close $fh;
    return $content;
}


1;

